/*
 * $Id: ngks.c,v 1.4 2001/10/17 13:14:52 kenta Exp $
 *
 * Copyright 2001 Kenta Cho. All rights reserved.
 */

/**
 * Nichangokushi main thread.
 *
 * @version $Revision: 1.4 $
 */
#include <string.h>
#include <stdio.h>
#include "kanji.h"
#include "functions.h"
#include "wkf.h"
#include "chadll.h"

#include "ngks.h"

#define CONV_SIZE_MAX 1024

static Conv conv[CONV_SIZE_MAX];
static int convSize;

#define WORD_LENGTH 64

#define DIC_FILE "ngks.dic"

static int getNextStr(FILE *fp, char *str) {
  int idx = 0;
  int c;
  for ( ; ; ) {
    c = fgetc(fp);
    switch ( c ) {
    case -1:
      return 0;
    case '/':
      str[idx] = '\0';
      return 1;
    case ';':
      str[idx] = '\0';
      return 2;
    }
    if ( c != '\n' && c != '\r' ) {
      str[idx] = c; idx++;
    }
  }
}

static void loadDic() {
  FILE *fp = fopen(DIC_FILE, "r");
  char str[WORD_LENGTH];
  int status, idx = 0;

  if ( fp == NULL ) {
    fprintf(stderr, "Unable to load count file.\n");
    return;
  }
  convSize = 0;
  for ( ; ; ) {
    status = getNextStr(fp, str);
    if ( status == 0 ) break;
    strcpy(conv[convSize].word[idx], str);
    status = getNextStr(fp, str);
    if ( status == 0 ) break;
    strcpy(conv[convSize].wordRuby[idx], str);
    status = getNextStr(fp, str);
    if ( status == 0 ) break;
    strcpy(conv[convSize].wordClass[idx], str);
    if ( status == 2 ) {
      status = getNextStr(fp, str);
      if ( status == 0 ) break;
      strcpy(conv[convSize].convertedWord, str);
      conv[convSize].length = idx+1;
      convSize++; idx = 0;
    } else {
      idx++;
    }
  }
  fclose(fp);
}

void initNgks() {
  loadDic();
}

static char convertResult[WORD_LENGTH*CONV_QUEUE_LENGTH];
static char convertBuf[CONV_QUEUE_LENGTH][WORD_LENGTH];
static int cbIdx;

static void resetConvert() {
  int i;
  for ( i=0 ; i<convSize ; i++ ) {
    conv[i].idx = conv[i].wcIdx = 0;
  }
  cbIdx = 0;
}

static char *convertWord(char *word, char *wordRuby, char *wordClass) {
  int i, j, ci, woi, wci, wcIdx;
  char *str;
  int noChange, checked = 0;
  Conv *cv;

  for ( i=0 ; i<convSize ; i++ ) {
    noChange = 0; 
    cv = &(conv[i]);
    ci = cv->idx;

    if ( cv->wordRuby[ci][0] != '*' ) {
      str = cv->wordRuby[ci];
      woi = 0;
      for ( wci = 0 ; wci < strlen(str) ; ) {
	if ( str[wci] == '?' ) {
	  if ( wordRuby[woi] == '\0' || wordRuby[woi+1] == '\0' ) {
	    noChange = 1;
	    break;
	  }
	  cv->wcChars[cv->wcIdx] = wordRuby[woi];
	  cv->wcChars[cv->wcIdx+1] = wordRuby[woi+1];
	  wci++; woi += 2; cv->wcIdx += 2;
	  continue;
	}
	if ( str[wci] != wordRuby[woi] || str[wci+1] != wordRuby[woi+1] ) {
	  noChange = 1;
	  break;
	}
	wci+=2; woi+=2;
      }
      if ( noChange || wordRuby[woi] != '\0' ) {
	cv->idx = cv->wcIdx = 0;
	continue;
      }
    }

    if ( cv->word[ci][0] != '*' ) {
      str = cv->word[ci];
      woi = 0;
      for ( wci = 0 ; wci < strlen(str) ; ) {
	if ( str[wci] == '?' ) {
	  if ( word[woi] == '\0' || word[woi+1] == '\0' ) {
	    noChange = 1;
	    break;
	  }
	  cv->wcChars[cv->wcIdx] = word[woi];
	  cv->wcChars[cv->wcIdx+1] = word[woi+1];
	  wci++; woi += 2; cv->wcIdx += 2;
	  continue;
	}
	if ( str[wci] != word[woi] || str[wci+1] != word[woi+1] ) {
	  noChange = 1;
	  break;
	}
	wci+=2; woi+=2;
      }
      if ( noChange || word[woi] != '\0' ) {
	cv->idx = cv->wcIdx = 0;
	continue;
      }
    }
    
    if ( cv->wordClass[ci][0] != '*' ) {
      if ( strstr(wordClass, cv->wordClass[ci]) == NULL ) {
	cv->idx = cv->wcIdx = 0;
	continue;
      }
    }

    cv->idx++;
    checked = 1;
    if ( cv->idx >= cv->length ) {
      convertResult[0] = '\0';
      for ( j=0 ; j<=cbIdx-cv->length ; j++ ) {
	strcat(convertResult, convertBuf[j]);
      }
      j = strlen(convertResult); wcIdx = 0;
      checked = 2; cbIdx = 0;
      for ( wci=0 ; wci<strlen(cv->convertedWord) ; ) {
	while ( cv->convertedWord[wci] == '+' ) {
	  wci++; wcIdx += 2;
	}
	while ( cv->convertedWord[wci] == '-' ) {
	  wci++; wcIdx -= 2;
	}
	if ( cv->convertedWord[wci] == '?' ) {
	  convertResult[j] = cv->wcChars[wcIdx];
	  convertResult[j+1] = cv->wcChars[wcIdx+1];
	  j += 2; wcIdx += 2; wci++;
	} else {
	  convertResult[j] = cv->convertedWord[wci];
	  convertResult[j+1] = cv->convertedWord[wci+1];
	  j += 2; wci += 2;
	}
      }
      convertResult[j] = '\0';
      //cv->idx = cv->wcIdx = 0;
      resetConvert();
      break;
    }
  }
  switch ( checked ) {
  case 0:
    convertResult[0] = '\0';
    for ( i=0 ; i<cbIdx ; i++ ) {
      strcat(convertResult, convertBuf[i]);
    }
    strcat(convertResult, word);
    cbIdx = 0;
    break;
  case 1:
    strcpy(convertBuf[cbIdx], word);
    cbIdx++;
    convertResult[0] = '\0';
    break;
  }
  return convertResult;
}

static char* getRemainedWords() {
  int i;
  convertResult[0] = '\0';
  for ( i=0 ; i<cbIdx ; i++ ) {
    strcat(convertResult, convertBuf[i]);
  }
  return convertResult;
}

static char tmpWords[BODY_BUF_SIZE];

static char* changeWords(char *str) {
  char *idx, *eIdx;
  char wordClass[WORD_LENGTH], wordRuby[WORD_LENGTH], word[WORD_LENGTH];
  char *convertedWord;
  kcode_t kcode, prvKcode = KC_SJIS;

  resetConvert();
  tmpWords[0] = '\0';
  idx = str;
  for ( ; ; ) {
    eIdx = idx;
    idx = strstr(idx, "/$;/");
    if ( idx == NULL || idx-eIdx >= WORD_LENGTH-2 ) break;
    strncpy(word, eIdx, idx-eIdx);
    word[idx-eIdx] = '\0';
    prvKcode = kcode;
    kcode = wkfGuessKanjiCodeOfString(word);

    idx += 4;
    eIdx = idx;
    idx = strstr(idx, "/@;/");
    if ( idx == NULL || idx-eIdx >= WORD_LENGTH-2 ) break;
    strncpy(wordRuby, eIdx, idx-eIdx);
    wordRuby[idx-eIdx] = '\0';

    idx += 4;
    eIdx = idx;
    idx = strstr(idx, "/#;/");
    if ( idx == NULL || idx-eIdx >= WORD_LENGTH-2 ) break;
    strncpy(wordClass, eIdx, idx-eIdx);
    wordClass[idx-eIdx] = '\0';
    idx += 4;

    if ( prvKcode == KC_ASCII && kcode == KC_ASCII ) {
      strcat(tmpWords, " ");
    } 
    strcat(tmpWords, convertWord(word, wordRuby, wordClass));
  }
  strcat(tmpWords, getRemainedWords());
  return tmpWords;
}

static char sjisBody[BODY_BUF_SIZE];
static char tmpBody[BODY_BUF_SIZE];
static char orgEncWords[BODY_BUF_SIZE];
static char changedBody[BODY_BUF_SIZE*2];

static kcode_t bodyKcode;

#define CHASEN_SIZE_CAPACITY 7680

static void changeSection(char *org, char *section, int *size) {
  char *changedWords;
  //kcode_t kcode = bodyKcode;
  kcode_t kcode = wkfGuessKanjiCodeOfString(org);
  wkfConvertKanjiCodeOfString(kcode, org, KC_SJIS, sjisBody, BODY_BUF_SIZE);
  if ( wkfGuessKanjiCodeOfString(sjisBody) == KC_ASCII 
       || strlen(sjisBody) > CHASEN_SIZE_CAPACITY ) {
    strcat(section, org);
    *size += strlen(org);
    return;
  }
  changedWords = changeWords(analyze(sjisBody));
  wkfConvertKanjiCodeOfString(KC_SJIS, changedWords, kcode, orgEncWords, BODY_BUF_SIZE);
  if ( strlen(orgEncWords) > 0 ) {
    strcat(section, orgEncWords);
    *size += strlen(orgEncWords);
  } else {
    strcat(section, changedWords);
    *size += strlen(changedWords);
  }
}

static changeBody(char *body, char **result, int *size) {
  char *idx = body, *pIdx;
  char *eIdx = body;
  int escaped = 0;
  *result = changedBody;
  changedBody[0] = '\0';
  *size = 0;
  for ( ; ; ) {
    pIdx = idx;
    if ( bodyKcode == KC_JIS ) {
      escaped = 0;
      for ( ; ; ) {
	idx = strchr(idx, '<');
	if ( idx == NULL || eIdx == NULL ) break;
	for ( ; ; ) {
	  eIdx = strchr(eIdx, 0x1b);
	  if ( eIdx == NULL || eIdx > idx ) {
	    break;
	  }
	  if ( (*(eIdx+1) == '$' && *(eIdx+2) == 'B') ||
	       (*(eIdx+1) == '$' && *(eIdx+2) == '@') ) {
	    escaped = 1;
	  } else if ( (*(eIdx+1) == '(' && *(eIdx+2) == 'B') ||
		      (*(eIdx+1) == '(' && *(eIdx+2) == 'J') ) {
	    escaped = 0;
	  }
	  eIdx++;
	}
	if ( eIdx == NULL || escaped == 0 ) break;
	idx++;
      }
    } else {
      idx = strchr(idx, '<');
    }

    if ( idx == NULL ) {
      changeSection(pIdx, changedBody, size);
      break;
    }
    if ( pIdx != idx ) {
      strncpy(tmpBody, pIdx, idx-pIdx);
      tmpBody[idx-pIdx] = '\0';
      changeSection(tmpBody, changedBody, size);
      pIdx = idx;
    }
    idx++;
    /*if ( bodyKcode == KC_JIS ) {
      for ( ; ; ) {
	idx = strchr(idx, '>');
	if ( idx == NULL || eIdx == NULL ) break;
	for ( ; ; ) {
	  eIdx = strchr(eIdx, 0x1b);
	  if ( eIdx == NULL || eIdx > idx ) {
	    break;
	  }
	  if ( (*(eIdx+1) == '$' && *(eIdx+2) == 'B') ||
	       (*(eIdx+1) == '$' && *(eIdx+2) == '@') ) {
	    escaped = 1;
	  } else if ( (*(eIdx+1) == '(' && *(eIdx+2) == 'B') ||
		      (*(eIdx+1) == '(' && *(eIdx+2) == 'J') ) {
	    escaped = 0;
	  }
	  eIdx++;
	}
	if ( eIdx == NULL || escaped == 0 ) break;
	idx++;
      }
    } else {
      idx = strchr(idx, '>');
    }*/
    idx = strchr(idx, '>');
    if ( idx == NULL ) break;
    idx++;
    strncpy(tmpBody, pIdx, idx-pIdx);
    tmpBody[idx-pIdx] = '\0';
    strcat(changedBody, tmpBody);
    *size += idx-pIdx;
  }
}

static int chasenArgc = 4;
static char *chasenArgv[4] = {"hoge", "-rchasenrc", "-j", "-F%m/$;/%y/@;/%U(%P-)/#;/"};

// Check a command from the user agent.
void hookProxyConn(char *hostname, char *pathname, char *gpc, char *referer, 
		   int isHtml, char *title, int bodySize,
		   char *buf, char **changedBuf, int *changedSize) {
  if ( !isHtml || bodySize == 0 ) {
    *changedBuf = buf; *changedSize = bodySize;
    return;
  }
  bodyKcode = wkfGuessKanjiCodeOfString(buf);
  printf("%s%s\n%s\n", hostname, pathname, title);
  set_argument_parameters(chasenArgc, chasenArgv);
  changeBody(buf, changedBuf, changedSize);
}
